/*
 * Copyright (C) 2005-2010 Andrey Kholmanskih. All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, write to the 
 *
 * Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330,
 * Boston, MA  USA  02111-1307
 * 
 * 
 * Andrey Kholmanskih
 * support@jdbreport.com
 */
package and.dbcomp;

import java.io.InputStreamReader;
import java.io.Serializable;
import java.net.URL;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.Driver;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;
import java.util.zip.DataFormatException;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;
import javax.swing.event.EventListenerList;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import and.dbcomp.dao.DaoException;
import and.properties.XMLStored;
import and.util.Compress;
import and.util.xml.XMLCoder;

/**
 * @version 1.2 22.10.2010
 * @author Andrey Kholmanskih
 * 
 */
public class DbSource implements JdbcSource, XMLStored, Serializable {

	private static final long serialVersionUID = 1L;

	private static final String PREFIX = "-4398070554958534145L";
	private static final String POSTFIX = "8005098017894178616L";

	private String alias;

	private String driverName;

	private String url;

	private String user;

	private String password;

	private Properties properties;

	private Connection connection;

	private String jndiName;

	private boolean autoCommit = false;
	private boolean inTransaction;

	protected transient EventListenerList listenerList = new EventListenerList();

	private DataSource dataSource;

	public DbSource() {
		super();
	}

	public DbSource(DataSource dataSource) {
		super();
		this.dataSource = dataSource;
	}

	public DbSource(Connection connection) {
		super();
		this.connection = connection;
	}

	public DataSource getDataSource() {
		return dataSource;
	}

	public void startTransaction() {
		inTransaction = true;
	}

	public void commitTransaction() throws SQLException {
		try {
			inTransaction = false;
			getConnection().commit();
		} catch (InstantiationException e) {
			throw new SQLException(e.getMessage());
		} catch (IllegalAccessException e) {
			throw new SQLException(e.getMessage());
		} catch (ClassNotFoundException e) {
			throw new SQLException(e.getMessage());
		} catch (NamingException e) {
			throw new SQLException(e.getMessage());
		}
	}

	public void rollbackTransaction() {
		try {
			inTransaction = false;
			getConnection().rollback();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (NamingException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	public boolean isTransaction() {
		return inTransaction;
	}

	/**
	 * @param alias
	 *            The alias to set.
	 */
	public void setAlias(String alias) {
		this.alias = alias;
	}

	/**
	 * @return Returns the alias.
	 */
	public String getAlias() {
		return alias;
	}

	/**
	 * @param driverName
	 *            The driverName to set.
	 */
	public void setDriverName(String driverName) {
		this.driverName = driverName;
	}

	/**
	 * @return Returns the driverName.
	 */
	public String getDriverName() {
		return driverName;
	}

	/**
	 * @param url
	 *            The url to set.
	 */
	public void setUrl(String url) {
		this.url = url;
	}

	/**
	 * @return Returns the url.
	 */
	public String getUrl() {
		return url;
	}

	/**
	 * @param user
	 *            The user to set.
	 */
	public void setUser(String user) {
		this.user = user;
	}

	/**
	 * @return Returns the user.
	 */
	public String getUser() {
		return user;
	}

	/**
	 * @param password
	 *            The password to set.
	 */
	public void setPassword(String password) {
		this.password = encodePassword(password);
	}

	/**
	 * @return Returns the password.
	 */
	public String getPassword() {
		return decodePassword(password);
	}

	/**
	 * @param properties
	 *            The properties to set.
	 */
	public void setProperties(Properties properties) {
		this.properties = properties;
	}

	/**
	 * @return Returns the properties.
	 */
	public Properties getProperties() {
		if (properties == null) {
			properties = new Properties();
		}
		return properties;
	}

	public void store(Element parent) {
		Element node = parent.getOwnerDocument().createElement("DBURL");
		storeAttributes(node);
		parent.appendChild(node);
	}

	/**
	 * @param parent
	 * @param node
	 */
	protected void storeAttributes(Element node) {
		node.setAttribute("alias", getAlias());
		node.setAttribute("jndiName", getJndiName());
		node.setAttribute("driver", getDriverName());
		node.setAttribute("url", getUrl());
		node.setAttribute("user", getUser());
		if (password != null)
			node.setAttribute("passwd", password);
		if (properties != null && properties.size() > 0) {
			for (Object key : properties.keySet()) {
				Element prop = node.getOwnerDocument()
						.createElement("property");
				String value = (String) properties.get(key);
				prop.setAttribute("name", key.toString());
				prop.setAttribute("value", value);
				node.appendChild(prop);
			}
		}
		node.setAttribute("autocommit", "" + isAutoCommit());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see and.properties.XMLStored#load(org.w3c.dom.Element)
	 */
	public void load(Element parent) {
		clear();
		NodeList nodes = parent.getElementsByTagName("DBURL");
		if (nodes == null)
			return;
		Element node = (Element) nodes.item(0);
		loadAttributes(node);
	}

	/**
	 * @param node
	 */
	protected void loadAttributes(Element node) {
		alias = node.getAttribute("alias");
		jndiName = node.getAttribute("jndiName");
		driverName = node.getAttribute("driver");
		url = node.getAttribute("url");
		user = node.getAttribute("user");
		password = node.getAttribute("passwd");
		if (password == null || password.length() == 0) {
			setPassword(node.getAttribute("password"));
		}
		if ("false".equals(node.getAttribute("autocommit")))
			autoCommit = false;
		NodeList props = node.getElementsByTagName("property");
		for (int i = 0; i < props.getLength(); i++) {
			Element el = (Element) props.item(i);
			properties.put(el.getAttribute("name"), el.getAttribute("value"));
		}
	}

	/**
	 * 
	 */
	protected void clear() {
		getProperties().clear();
		alias = "";
		jndiName = "";
		driverName = "";
		url = "";
		user = "";
		password = "";
		autoCommit = true;
	}

	public String toString() {
		return alias;
	}

	public boolean isConnected() {
		try {
			return (connection != null && !connection.isClosed());
		} catch (SQLException e) {
			e.printStackTrace();
			return false;
		}
	}

	public Connection createConnection() throws SQLException,
			InstantiationException, IllegalAccessException,
			ClassNotFoundException, NamingException {

		if (dataSource == null && getJndiName() != null
				&& !"".equals(getJndiName())) {
			try {
				InitialContext ctx = new InitialContext();
				dataSource = (DataSource) ctx.lookup(getJndiName());
			} catch (NamingException e) {
				if (driverName == null) {
					throw e;
				}
				e.printStackTrace();
			}
		}

		if (dataSource != null) {
			if (user != null && user.length() > 0)
				return dataSource.getConnection(user, getPassword());
			else
				return dataSource.getConnection();
		}

		java.sql.DriverManager.registerDriver((java.sql.Driver) Class.forName(
				driverName).newInstance());
		java.util.Properties connectionProperties = new java.util.Properties();
		if (user != null && user.length() > 0) {
			connectionProperties.put("user", user);
		}
		String p = getPassword();
		if (p != null && p.length() > 0) {
			connectionProperties.put("password", p);
		}
		if (properties != null) {
			for (Object key : properties.keySet()) {
				String value = properties.get(key).toString();
				if (value != null && value.length() > 0)
					connectionProperties.put(key, value);
			}
		}
		Driver d = java.sql.DriverManager.getDriver(url);
		Connection conn = d.connect(url, connectionProperties);
		conn.setAutoCommit(isAutoCommit());
		setAutoCommit(conn.getAutoCommit());
		if (conn.getMetaData().supportsTransactionIsolationLevel(
				Connection.TRANSACTION_READ_COMMITTED)) {
			conn.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED);
		}
		return conn;
	}

	public Connection getConnection() throws SQLException,
			InstantiationException, IllegalAccessException,
			ClassNotFoundException, NamingException {
		if (connection == null || connection.isClosed()) {
			connection = createConnection();
		}
		return connection;
	}

	protected void setConnection(Connection connection) {
		this.connection = connection;
	}

	public DatabaseMetaData getMetaData() throws SQLException {
		return connection.getMetaData();
	}

	/**
	 * @param autoCommit
	 *            The autoCommit to set.
	 */
	public void setAutoCommit(boolean autoCommit) {
		this.autoCommit = autoCommit;
	}

	/**
	 * @return Returns the autoCommit.
	 */
	public boolean isAutoCommit() {
		return autoCommit;
	}

	public void connect() throws SQLException, InstantiationException,
			IllegalAccessException, ClassNotFoundException, NamingException {

		try {
			if (connection == null || connection.isClosed()) {
				connection = createConnection();
			}
		} catch (SQLException e) {
			try {
				disconnect();
			} catch (SQLException e1) {
			}
			throw e;
		} catch (InstantiationException e) {
			try {
				disconnect();
			} catch (SQLException e1) {
			}
			throw e;
		} catch (IllegalAccessException e) {
			try {
				disconnect();
			} catch (SQLException e1) {
			}
			throw e;
		} catch (ClassNotFoundException e) {
			try {
				disconnect();
			} catch (SQLException e1) {
			}
			throw e;
		} catch (NamingException e) {
			try {
				disconnect();
			} catch (SQLException e1) {
			}
			throw e;
		}
	}

	public void disconnect() throws SQLException {
		fireConnectionClose();
		Connection tmpConnection = connection;
		connection = null;
		if (tmpConnection != null && !tmpConnection.isClosed()) {
			try {
				tmpConnection.rollback();
			} catch (Exception e) {
			}
			tmpConnection.close();
		}
	}

	/**
	 * @param jndiName
	 *            the jndiName to set
	 */
	public void setJndiName(String jndiName) {
		this.jndiName = jndiName;
	}

	/**
	 * @return the jndiName
	 */
	public String getJndiName() {
		return jndiName;
	}

	private String encodePassword(String pass) {
		if (pass == null) {
			return null;
		}
		StringBuffer tmp = new StringBuffer();
		tmp.append(PREFIX);
		tmp.append(pass);
		tmp.append(POSTFIX);
		tmp.append(tmp);
		byte[] b;
		b = Compress.compress(tmp.toString().getBytes());
		for (int i = 0; i < b.length; i++) {
			b[i] = (byte) ~b[i];
		}
		b = XMLCoder.base64Encode(b);
		String result = new String(b);
		return result;
	}

	private String decodePassword(String pass) {
		if (pass == null) {
			return null;
		}
		byte[] b = pass.getBytes();
		try {
			b = XMLCoder.base64Decode(b);
		} catch (Exception e) {
			e.printStackTrace();
			return pass;
		}
		for (int i = 0; i < b.length; i++) {
			b[i] = (byte) ~b[i];
		}
		try {
			b = Compress.decompress(b);
		} catch (DataFormatException e1) {
			e1.printStackTrace();
		}
		String result = new String(b);
		result = result.substring(0, result.length() / 2);
		result = result.substring(PREFIX.length(),
				result.length() - POSTFIX.length());
		return result;
	}

	public void addConnectionListener(ConnectionListener x) {
		listenerList.add(ConnectionListener.class, x);
	}

	public void removePeriodListener(ConnectionListener x) {
		listenerList.remove(ConnectionListener.class, x);
	}

	protected void fireConnectionClose() {
		ConnectionEvent e = new ConnectionEvent(this);
		Object[] listeners = listenerList.getListenerList();
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == ConnectionListener.class) {
				((ConnectionListener) listeners[i + 1]).connectionClose(e);
			}
		}
	}

	public static void execScript(Connection connection, URL url)
			throws DaoException {
		execScript(connection, url, "UTF-8", "^");
	}

	public static void execScript(Connection connection, URL url, String charSet)
			throws DaoException {
		execScript(connection, url, charSet, "^");
	}

	public static void execScript(Connection connection, URL url,
			String charSet, String delimiter) throws DaoException {
		if (url == null)
			return;
		try {
			if (charSet == null) {
				charSet = "UTF-8";
			}
			if (delimiter == null) {
				delimiter = "^";
			}
			InputStreamReader reader = new InputStreamReader(url.openStream(),
					charSet);
			int bufSize = 16384;
			try {
				Statement stat = connection.createStatement();
				StringBuffer script = new StringBuffer();
				int l = 0;
				char[] buf = new char[bufSize];
				do {
					l = reader.read(buf);
					if (l > 0) {
						script.append(new String(buf, 0, l));
						findAndExecSql(connection, delimiter, stat, script);
					}
				} while (l == bufSize);
				findAndExecSql(connection, delimiter, stat, script);
				if (script.length() > 0) {
					String sql = script.toString().trim();
					if (sql.length() > 0) {
						if (sql.equalsIgnoreCase("COMMIT")
								|| sql.equalsIgnoreCase("COMMIT WORK")) {
							connection.commit();
						} else {
							stat.execute(sql);
						}
					}
				}
			} finally {
				reader.close();
			}
		} catch (DaoException e) {
			throw e;
		} catch (Exception e) {
			throw new DaoException(e);
		}
	}

	protected static void findAndExecSql(Connection connection,
			String delimiter, Statement stat, StringBuffer script)
			throws DaoException {
		int i = 0;
		String sql = null;
		try {
			do {
				i = script.indexOf(delimiter);
				if (i > 0) {
					sql = script.substring(0, i).trim();
					script.delete(0, i + 1);
					if (sql != null && sql.length() > 0) {
						if (sql.equalsIgnoreCase("COMMIT")
								|| sql.equalsIgnoreCase("COMMIT WORK")) {
							connection.commit();
						} else {
							stat.execute(sql);
						}

					}
				}
			} while (i > 0);
		} catch (Exception e) {
			System.out.println("SQL : " + sql);
			throw new DaoException(e);
		}
	}

}
